# -*- coding: utf-8 -*-

import os
import json
from collections import Counter
from datetime import datetime, timedelta
from typing import Dict, List, Tuple

from ydl_ai_recommender.src.utils import get_data_path
from ydl_ai_recommender.src.utils.log import create_logger
from ydl_ai_recommender.src.core.manager import OrderDataManager, ChatDataManager


class Indexer():
    """
    索引构建、管理类
    """
    def __init__(self, logger=None) -> None:
        if logger is None:
            self.logger = create_logger(__name__)
        else:
            self.logger = logger

        self.local_file_dir = get_data_path()


    def index(self, q: str, count: int=0) -> List[Tuple[str, float]]:
        """
        返回值类型：[[相似id, score], [相似id, score], ...]
        """
        raise NotImplementedError
    
    def make_index(self) -> Dict[str, List]:
        raise NotImplementedError


class UserCounselorDefaultIndexer(Indexer):
    """
    [用户->咨询师]兜底关系索引
    """
    def __init__(self, logger=None) -> None:
        super().__init__(logger)
        self.data_manager = OrderDataManager(logger)
        self.index_file = os.path.join(self.local_file_dir, 'index_list.txt')
        self.count = 100
        self.index_data = []

    def load_index_data(self):
        index = []
        with open(self.index_file, 'r', encoding='utf-8') as f:
            index = [(line.strip(), 0.01 - 0.0001 * index) for index, line in enumerate(f)]
        self.index_data = index

    def index(self, q='', count=0) -> List[Tuple[str, float]]:
        if len(self.index_data) == 0:
            self.logger.error('未加载索引数据，使用`index`函数之前，确认对应已执行执行 `load_index_data()` 方法')
            raise

        if count == 0:
            return self.index_data
        else:
            return self.index_data[:count]

    def make_index(self) -> Dict[str, List]:
        self.logger.info('')
        self.logger.info('开始构建[用户->咨询师]兜底关系索引')
        
        df = self.data_manager.load_raw_data()
        self.logger.info('构建索引数据加载完成，共加载 %s 条数据', len(df))

        supplier_cnter = Counter(df['supplier_id'])
        index_list = []
        for key, _ in supplier_cnter.most_common(self.count):
            index_list.append(str(key))

        self.logger.info('[用户->咨询师]兜底关系索引构建完成')
        with open(self.index_file, 'w', encoding='utf-8') as f:
            f.write('\n'.join(index_list))
        self.logger.info('[用户->咨询师]兜底关系索引构建完成已保存')
        
        return index_list


class UserCounselorOrderIndexer(Indexer):
    """
    基于订单数据的[用户->咨询师]关系索引
    """

    def __init__(self, logger=None) -> None:
        super().__init__(logger)
        self.data_manager = OrderDataManager(logger)
        self.index_file = os.path.join(self.local_file_dir, 'user_counselor_order_index.json')
        self.index_data = {}
        self.now = datetime.now()

    def _compute_score(self, infos: List) -> float:
        w = [0, 0, 0, 0]
        for [_price, dt] in infos:
            price = float(_price)
            date = datetime.strptime(dt, '%Y-%m-%d')

            if (self.now - date) <= timedelta(days=7):
                w[0] = max(1., w[0], price / 400)
            elif (self.now - date) <= timedelta(days=30):
                w[1] = max(1., w[1], price / 400)
            elif (self.now - date) <= timedelta(days=180):
                w[2] = max(1., w[2], price / 400)
            else:
                w[3] = max(1., w[3], price / 400)

        value = w[0] * 0.5 + w[1] * 0.25 + w[2] * 0.15 + w[3] * 0.1
        return value

    def make_index(self) -> Dict[str, List]:
        self.logger.info('')
        self.logger.info('开始构建基于订单数据的[用户->咨询师]关系索引')
        
        df = self.data_manager.load_raw_data()
        self.logger.info('构建索引数据加载完成，共加载 %s 条数据', len(df))

        user_order = {}
        for index, row in df.iterrows():
            uid, supplier_id = row['uid'], row['supplier_id']
        
            if uid not in user_order:
                user_order[uid] = {}
            if supplier_id not in user_order[uid]:
                user_order[uid][supplier_id] = []
            user_order[uid][supplier_id].append([row['price'], row['update_time']])
        
        index = {}
        for uid, orders in user_order.items():
            supplier_values = []
            for supplier_id, infos in orders.items():
                value = self._compute_score(infos)
                supplier_values.append((supplier_id, value))
            index[uid] = sorted(supplier_values, key=lambda x: x[1], reverse=True)
        
        self.logger.info('基于订单数据的[用户->咨询师]关系索引构建完成，共构建 %s 条数据', len(index))
        with open(self.index_file, 'w', encoding='utf-8') as f:
            json.dump(index, f, ensure_ascii=False)

        self.logger.info('基于订单数据的[用户->咨询师]关系索引数据已保存，共有用户 %s', len(index))

    def load_index_data(self):
        index = []
        with open(self.index_file, 'r', encoding='utf-8') as f:
            index = json.load(f)
        self.index_data = index

    def index(self, q='', count=0) -> List[Tuple[str, float]]:
        if len(self.index_data) == 0:
            self.logger.error('未加载索引数据，使用`index`函数之前，确认对应已执行执行 `load_index_data()` 方法')
            raise

        if count == 0:
            return self.index_data.get(q, [])
        else:
            return self.index_data.get(q, [])[:count]


class UserCounselorChatIndexer(Indexer):
    """
    基于询单数据的[用户->咨询师]关系索引
    """

    def __init__(self, logger=None) -> None:
        super().__init__(logger)
        self.data_manager = ChatDataManager(logger)
        self.index_file = os.path.join(self.local_file_dir, 'user_counselor_chat_index.json')
        self.index_data = {}
        self.now = datetime.now()

    def _compute_score(self, infos: List) -> float:
        w = [0, 0, 0, 0]
        for [dt, _u2d, _d2u] in infos:
            u2d, d2u = int(_u2d), int(_d2u)
            date = datetime.strptime(dt, '%Y-%m-%d')

            if (self.now - date) <= timedelta(days=7):
                w[0] = max(1., w[0], (u2d + d2u) / 20)
            elif (self.now - date) <= timedelta(days=30):
                w[1] = max(1., w[1], (u2d + d2u) / 20)
            elif (self.now - date) <= timedelta(days=180):
                w[2] = max(1., w[2], (u2d + d2u) / 20)
            else:
                w[3] = max(1., w[3], (u2d + d2u) / 20)

        value = w[0] * 0.5 + w[1] * 0.25 + w[2] * 0.15 + w[3] * 0.1
        return value

    def make_index(self) -> Dict[str, List]:
        self.logger.info('')
        self.logger.info('开始构建基于询单数据的[用户->咨询师]关系索引')
        
        df = self.data_manager.load_raw_data()
        self.logger.info('构建索引数据加载完成，共加载 %s 条数据', len(df))

        user_chat = {}
        for index, row in df.iterrows():
            uid, supplier_id = row['uid'], row['doctor_id']
        
            if uid not in user_chat:
                user_chat[uid] = {}
            if supplier_id not in user_chat[uid]:
                user_chat[uid][supplier_id] = []

            user_chat[uid][supplier_id].append([row['dt'], row['user_to_doctor'], row['doctor_to_user']])

        index = {}
        for uid, chats in user_chat.items():
            supplier_values = []
            for supplier_id, infos in chats.items():
                value = self._compute_score(infos)
                supplier_values.append([supplier_id, value])
            index[uid] = sorted(supplier_values, key=lambda x: x[1], reverse=True)
        
        self.logger.info('基于询单数据的[用户->咨询师]关系索引构建完成，共构建 %s 条数据', len(index))
        with open(self.index_file, 'w', encoding='utf-8') as f:
            json.dump(index, f, ensure_ascii=False)

        self.logger.info('基于询单数据的[用户->咨询师]关系索引数据已保存，共有用户 %s', len(index))

    def load_index_data(self):
        index = []
        with open(self.index_file, 'r', encoding='utf-8') as f:
            index = json.load(f)
        self.index_data = index
    

    def index(self, q='', count=0) -> List[Tuple[str, float]]:
        if len(self.index_data) == 0:
            self.logger.error('未加载索引数据，使用`index`函数之前，确认对应已执行执行 `load_index_data()` 方法')
            raise

        if count == 0:
            return self.index_data.get(q, [])
        else:
            return self.index_data.get(q, [])[:count]


class UserCounselorCombinationIndexer(Indexer):
    """
    基于多种数据组合的[用户->咨询师]关系索引
    """
    def __init__(self, order_w=0.6, chat_w=0.4, logger=None) -> None:
        super().__init__(logger)
        self.order_w = order_w
        self.chat_w = chat_w
        self.index_file = os.path.join(self.local_file_dir, 'user_counselor_combination_index.json')
        self.index_data = {}

    def make_index(self) -> Dict[str, List]:
        self.logger.info('')
        self.logger.info('开始构建基于多种数据组合的[用户->咨询师]关系索引')

        order_indexer = UserCounselorOrderIndexer(self.logger)
        chat_indexer = UserCounselorChatIndexer(self.logger)
        order_indexer.load_index_data()
        chat_indexer.load_index_data()
        self.logger.info('构建索引数据加载完成')

        index = {}
        for uid, counselors in order_indexer.index_data.items():
            chat_index = {
                c_id: value for c_id, value in chat_indexer.index_data.get(uid, [])
            }
            new_counselors = []
            for (c_id, value) in counselors:
                merge_value = value * self.order_w + chat_index.get(c_id, 0) * self.chat_w
                new_counselors.append((c_id, merge_value))

            index[uid] = sorted(new_counselors, key=lambda x: x[1], reverse=True)

        self.logger.info('基于多种数据组合的[用户->咨询师]关系索引构建完成，共构建 %s 条数据', len(index))
        with open(self.index_file, 'w', encoding='utf-8') as f:
            json.dump(index, f, ensure_ascii=False)
        
        return index

    def load_index_data(self):
        index = []
        with open(self.index_file, 'r', encoding='utf-8') as f:
            index = json.load(f)
        self.index_data = index
    
    def index(self, q='', count=0) -> List[Tuple[str, float]]:
        if len(self.index_data) == 0:
            self.logger.error('未加载索引数据，使用`index`函数之前，确认对应已执行执行 `load_index_data()` 方法')
            raise

        if count == 0:
            return self.index_data.get(q, [])
        else:
            return self.index_data.get(q, [])[:count]


if __name__ == '__main__':
    indexer = UserCounselorDefaultIndexer()
    indexer.make_index()

    indexer = UserCounselorOrderIndexer()
    indexer.make_index()

    indexer = UserCounselorChatIndexer()
    indexer.make_index()

    indexer = UserCounselorCombinationIndexer()
    indexer.make_index()