# -*- coding: utf-8 -*-

import os
import json
from typing import List

import pandas as pd

from ydl_ai_recommender.src.core.profile import encode_profile
from ydl_ai_recommender.src.core.manager import DatabaseDataManager
from ydl_ai_recommender.src.utils.log import create_logger


class ProfileManager(DatabaseDataManager):
    """
    订单用户画像数据管理
    """

    def __init__(self, client=None) -> None:
        super().__init__(client, create_logger(__name__, 'profile_manager.log'))


    def _make_query_sql(self, conditions=None):
        condition_sql = ''
        if conditions:
            condition_sql = ' WHERE ' + ' AND '.join(conditions)

        sql = 'SELECT * FROM ads.ads_register_user_profiles'
        sql += ' WHERE uid IN (SELECT DISTINCT uid FROM ods.ods_ydl_standard_order{})'.format(condition_sql)
        return sql

    def update_data(self):
        """ 从数据库中拉取最新画像特征并保存 """

        sql = self._make_query_sql()
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_xlsx_data(df, 'all_profile.xlsx')

    def update_test_data(self, conditions):
        """ 从数据库中拉取指定条件画像信息用于测试 """
        
        sql = self._make_query_sql(conditions)
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_xlsx_data(df, 'test_profile.xlsx')

    def _load_profile_data(self):
        return self.load_xlsx_data('all_profile.xlsx')

    def load_test_profile_data(self):
        return self.load_xlsx_data('test_profile.xlsx')

    def make_embeddings(self):
        user_profiles = self._load_profile_data()
        self.logger.info('订单用户画像数据加载完成，共加载 %s 条', len(user_profiles))
        user_ids, embeddings = [], []
        self.logger.info('开始构建订单用户的用户画像向量')
        for _, profile in user_profiles.iterrows():
            user_ids.append(str(profile['uid']))
            embeddings.append(encode_profile(profile))
        
        self.logger.info('用户画像向量构建完成，共构建 %s 用户', len(user_ids))

        with open(os.path.join(self.local_file_dir, 'user_embeddings_ids.txt'), 'w', encoding='utf-8') as f:
            f.write('\n'.join(user_ids))

        with open(os.path.join(self.local_file_dir, 'user_embeddings.json'), 'w', encoding='utf-8') as f:
            json.dump(embeddings, f, ensure_ascii=False)

        return embeddings

    def make_virtual_embedding(self):
        user_ids = []
        embeddings = []

        with open(os.path.join(self.local_file_dir, 'user_embeddings_ids.txt'), 'r', encoding='utf-8') as f:
            user_ids = [line.strip() for line in f]

        with open(os.path.join(self.local_file_dir, 'user_embeddings.json'), 'r', encoding='utf-8') as f:
            embeddings = json.load(f)

        v_embedding_set = {}
        for user_id, embedding in zip(user_ids, embeddings):
            key = '_'.join(map(str, embedding))
            if key not in v_embedding_set:
                v_embedding_set[key] = {
                    'embedding': embedding,
                    'user_ids': [],
                }
            v_embedding_set[key]['user_ids'].append(str(user_id))
        
        v_embedding_list = []
        with open(os.path.join(self.local_file_dir, 'virtual_user_embeddings_ids.txt'), 'w', encoding='utf-8') as f:
            for info in v_embedding_set.values():
                f.write(','.join(info['user_ids']) + '\n')
                v_embedding_list.append(info['embedding'])

        with open(os.path.join(self.local_file_dir, 'virtual_user_embeddings.json'), 'w', encoding='utf-8') as f:
            json.dump(v_embedding_list, f, ensure_ascii=False)


if __name__ == '__main__':
    manager = ProfileManager()
    # manager.make_embeddings()
    manager.make_virtual_embedding()